/*
 * fservprm.c - Simple File Server Config File and Permissions functions
 *
 * Copyright (c) 2000 Cogito, LLC 
 *
 * Cogito grants you ("Licensee") a non-exclusive, royalty free, license to
 * use, modify and redistribute this software in source and binary form,
 * provided that i) this copyright notice and license appear on all copies
 * of the software; and ii) Licensee does not utilize the software in a
 * manner which is disparaging to Cogito.
 *
 * This software is provided "AS IS," without warranty of any kind.
 *
 * 12/2000: Initial version.
 *          M. Tim Jones <mtj@cogitollc.com>
 *
 */

#include <stdio.h>
#include <fcntl.h>
#include <dirent.h>
#include <sys/stat.h>
#include <arpa/inet.h>
#include "fservprm.h"

static authTableType authTable[MAX_AUTH_ELEMENTS];
static authRecords = 0;

/* Clear out the authorization table (no entries)
 */
void clearAuthTable(void)
{
  authRecords = 0;
  bzero(authTable, sizeof(authTable));
}


/* Read the contents of the external configuration file and store it within
 * the internal authorization table.
 */
int readConfigFile(void)
{
  int i, ret;
  int lineno = 0;
  char line[sizeof(authTableType)+1];
  char temp[4];
  FILE *fp;

  if ((fp = fopen(CONFIG_FILE, "r")) == NULL) {
    printf("Unable to open configuration file (%s)\n", CONFIG_FILE);
    return -1;
  }

  clearAuthTable();

  while( fgets(line, sizeof(line), fp) != NULL) {

    lineno++;

    if ((line[0] == 0x0a) || (line[0] == 0x0d)) {
      continue;
    } else if (line[0] != '#') {

      ret = sscanf(line, "%s %s %3s", 
                    &authTable[authRecords].ipAddress,
                    &authTable[authRecords].subdir,
                    &temp);

      // Were the proper number of elements found?
      if (ret != 3) {
        printf("Insufficient elements in configuration file (line %d)\n", 
               lineno);
        return -1;
      }

      // Does the subdirectory exist?
      if (opendir(authTable[authRecords].subdir) == NULL) {
        printf("Bad directory spec at line %d\n", lineno);
        return -1;
      }

      // Ensure that the subdir ends with '/'
      if (authTable[authRecords].  
            subdir[strlen(authTable[authRecords].subdir)-1] != '/') {
        strcat(authTable[authRecords].subdir, "/\0");
      }

      // Were the permissions correct?
      for (i = 0 ; i < sizeof(temp) ; i++) {
        if (tolower(temp[i]) == 'r') {
          authTable[authRecords].permissions |= PERM_R;
        } else if (tolower(temp[i]) == 'w') {
          authTable[authRecords].permissions |= PERM_W;
        } else if ((temp[i] == 0x0d) || (temp[i] == 0x0a) || (temp[i] == 0)) {
          break;
        } else {
          printf("Bad permissions on line %d\n", lineno);
          return -1;
        }
      }

      authRecords++;

    }

  }

  printf("Retrieved %d records from %s\n", authRecords, CONFIG_FILE);
  
  return 0;
}


/* Authentication is simply the process of identifying whether an IP address
 * exists in the table.
 */
int authenticateClient(char *client)
{
  int i;

  for (i = 0 ; i < authRecords ; i++) {
    if (!strcmp(authTable[i].ipAddress, client)) {
      return 1;
    }
  }
  
  return -1;
}


/* This function takes a string IP address as an argument and passes back
 * the home path for the particular user.
 */
char *getSubdir(char *client)
{
  int i;

  for (i = 0 ; i < authRecords ; i++) {
    if (!strcmp(authTable[i].ipAddress, client)) {
      return authTable[i].subdir;
    }
  }
  
  return NULL;
}

